//dynare code: Originally written by Pat Higgins.
//For Liu, Wang, and Zha: Land-price dynamics and macroeconomic fluctuations.
//
//-------------------------------------
//Shocks identified in the model
//-------------------------------------
// Eps_a:         Intertemporal preference (risk premium) shock
// Eps_phi:       Housing demand shock
// Eps_psi:       Labor supply shock
// Eps_z:         Permanent neutral technology shock
// Eps_v:         Transitory neutral technology shock
// Eps_q:         Permanent shock to investment specific technical change
// Eps_mu:        Transitory shock to investment specific technical change
// Eps_xitheta:   Collateral shock


//----------------------------------------------
//--  Declare varaibles, shocks,  and parameter 
//----------------------------------------------

var 
//--- 19 Endogenous state variables
Muh W Ql R Mue Mub N I Y Ch Ce Qk Lh Le K B G_Lambda G_Z G_Q
//--- 8 variables appearing in shock pricesses
Lambda_z V Lambda_q Mu Lambda_a Phi Psi Thetat
//--- 6 Observed data
DLogQl DLogQ  DLogC  DLogI  DLogB  LogL
//--- Additional (auxillary) variables
C DLogGDP
;

varexo Eps_a Eps_phi Eps_psi Eps_z Eps_v Eps_q  Eps_mu  Eps_xitheta;

parameters
//--- 5 estimated parameters
gamma_h gamma_e omega g_trans lambda_qbar_trans
//--- 8 calibrated parameters
alpha r ky ik qlLeOY theta_bar qlLhOY n lbar
//--- 8 parameters related to shock processes
rho_z rho_v rho_q rho_mu rho_a rho_phi rho_psi rho_xitheta;


//----------------------------------------------------------------
//-- Calibrate parameters to check steady state
//----------------------------------------------------------------
//--- Persistence parameters
//rho_a = 0.9108;
//rho_z = 0.4743;
//rho_v = 0.0074;
//rho_q = 0.6078;
//rho_mu = 0.2920;
//rho_phi = 0.9998;
//rho_psi = 0.9799;
//rho_xitheta = 0.9790;
rho_a = 0;
rho_z = 0;
rho_v = 0;
rho_q = 0;
rho_mu = 0;
rho_phi = 0;
rho_psi = 0;
rho_xitheta = 0;
//--- Deep parameters 
gamma_h =     0.7; %//<<>>Patrick change, gamma_h = 0.4655;
gamma_e =     0.7; %//<<>>Patrick change, gamma_e = 0.6050;
omega =      2; %//<<>>Patrick change, omega = 0.1881;
g_trans = 100*(1.00375-1); %//<<>>Patrick change, g_trans = 0.3682;
lambda_qbar_trans = 1.00; %//<<>>Patrick change, lambda_qbar_trans = 1.2530;
//--- Targeted steady state values
alpha = 0.30; //(1) the average labor income share is 70% (page 15 of paper)          
r = 1.01; //(2) the average real prime loan rate is 4% per annum (page 15 of paper)
ky = 4.6194; //(3) the capital-output ratio is on average 1.15 at the annual frequency (page 15 of paper)
ik = 0.2093/4; // (4) the investment-capital ratio is on average 0.209 at the annual frequency  
qlLeOY = 2.60; //(5) the average land-output ratio is 0.65 at the annual frequency
theta_bar = 0.75;  //(6) the average nonfarm and nonfiancial businesses' loan-asset ratio is 0.75 at the annual frequency (page 15 of paper)
qlLhOY = 5.8011; //(7) the average housing output ratio is 1.45 at the annual frequency
n = 1/4; //(8) average market hours is 25% of time endowment
lbar = 1; //arbitrary



//--------------------------------------------------------------------
//-- Model specification
//--------------------------------------------------------------------
model(linear);
# g = g_trans/100 + 1;
# lambda_qbar = 0.01*lambda_qbar_trans + 1;
# lambda_kbar = g*lambda_qbar; //See second sentence of first full paragraph of page 9
# delta = 1-(1-ik)*lambda_kbar ;  //Rearrange equation 60 

//From Tao's calibration5.m file
# betaparam = (qlLeOY*(1.0-g*theta_bar/r)+ky*(1.0-g*theta_bar/(lambda_kbar*r)))/(alpha+qlLeOY*(1.0-theta_bar)+ky*(1.0-delta-theta_bar)/lambda_kbar); 
# lambda_abar = g/(betaparam*r)-1.0; //See equation 58 (first part)
# muboe = (betaparam*lambda_abar)/g; //See equation 58 (second part)
# phi = qlLeOY*(1.0-betaparam*(1.0+lambda_abar*theta_bar))/(betaparam*alpha); // from calibration5.m
// # qlLeOY = (betaparam*alpha*phi)/(1 - betaparam - betaparam*lambda_abar*theta_bar) //Equation 59;
//Do not need equation 61

# iy = ik*ky; //Equation 62
# by = theta_bar*g*qlLeOY + (theta_bar/lambda_qbar)*ky; //Equation 63


# ceOY = alpha-iy-by*(1-betaparam*(1+lambda_abar))/g; //See equation 64
# chOY = 1-ceOY-iy; //See equation 65

# lhOle = qlLhOY/qlLeOY; //See equation 67
# le = lbar/(1.0+lhOle);
# lh = lbar-le;

# psi_bar = ((1-alpha)*g*(1-gamma_h/r)*(1/chOY))/(n*(g-gamma_h)); //Equation 68 

# omegah = (g - betaparam*(1+lambda_abar)*gamma_h)*(g - gamma_h); //See top of page 12
# omegae = (g - betaparam*gamma_e)*(g - gamma_e); //See top of page 12

//Patrick note, replaced G_Lambda_star with G_Lambda in eqn 59 of WangALinearized022809.dyn 
//(1)
omegah*Muh = -(g^2 + (gamma_h^2)*betaparam*(1+lambda_abar))*Ch + g*gamma_h*(Ch(-1) - G_Lambda) -
betaparam*lambda_abar*gamma_h*(g - gamma_h)*Lambda_a(+1) + 
betaparam*(1+lambda_abar)*g*gamma_h*(Ch(+1) + G_Lambda(+1)); //Eqn (69)


//Patrick note, deleted eta*N term in WangALinearized022809.dyn 
//(2)
W + Muh = Psi ; // Eqn. 70

//Patrick note, made slight changes to Eqn 61 in WangALinearized022809.dyn 
//(3)
Ql + Muh = betaparam*(1+lambda_abar)*(Muh(+1) + Ql(+1)) + (1 - betaparam*(1+lambda_abar))*(Phi -
Lh) + betaparam*lambda_abar*Lambda_a(+1); // Eqn. 71


//Patrick note, replaced G_Lambda_star with G_Lambda in eqn 62 of WangALinearized022809.dyn
//(4) 
Muh - R = Muh(+1) + (lambda_abar/(1+lambda_abar))*Lambda_a(+1) - G_Lambda(+1); // Eqn (72)

//Patrick note, replaced G_Lambda_star with G_Lambda in eqn 63 of WangALinearized022809.dyn 
//(5)
omegae*Mue = -(g^2 + betaparam*gamma_e^2)*Ce + g*gamma_e*(Ce(-1) - G_Lambda) +
betaparam*g*gamma_e*(Ce(+1) + G_Lambda(+1)); // Eqn (73)

//(6)
W = Y - N; // Eqn (74)

//Patrick note, replaced G_Lambda_star with G_Lambda in eqn 75 of WangALinearized022809.dyn 
//Also substituted in lambda_kbar
//(7)
Qk = (1+betaparam)*omega*(lambda_kbar^2)*I - omega*(lambda_kbar^2)*I(-1) +
omega*(lambda_kbar^2)*(G_Lambda + G_Q) - betaparam*omega*(lambda_kbar^2)*(I(+1)
 + G_Lambda(+1) + G_Q(+1)); // Eqn (75)

//Deleted thetak
//(8)
Qk + Mue = (muboe*theta_bar/lambda_qbar)*(Mub + Thetat) + 
(betaparam*(1-delta)/(lambda_kbar))*(Qk(+1) - G_Q(+1) - G_Lambda(+1)) + 
(1 - (muboe*theta_bar/lambda_qbar))*Mue(+1) + 
(muboe*theta_bar/lambda_qbar)*(Qk(+1) - G_Q(+1)) + 
betaparam*alpha*(1 - phi)*(1/ky)*(Y(+1) - K); // Eqn (76)

//(9)
Ql + Mue = muboe*g*theta_bar*(Thetat + Mub) + (1 - muboe*g*theta_bar)*Mue(+1) + 
muboe*g*theta_bar*(Ql(+1) + G_Lambda(+1)) + betaparam*Ql(+1) + 
(1 - betaparam - betaparam*lambda_abar*theta_bar)*(Y(+1) - Le); // Eqn (77)

//(10)
Mue - R = (1/(1+lambda_abar))*(Mue(+1) - G_Lambda(+1) + lambda_abar*Mub); // Eqn (78)

//(11)
Y = alpha*phi*Le(-1) + alpha*(1- phi)*K(-1) + (1-alpha)*N - 
(((1-phi)*alpha)/(1-((1-phi)*alpha)))*(G_Z + G_Q); // Eqn (79)

//(12)
K = ((1-delta)/(lambda_kbar))*(K(-1) - G_Lambda - G_Q) 
+ (1 - ((1-delta)/(lambda_kbar)))*I; // Eqn (80)

//(13)
Y = chOY*Ch + ceOY*Ce + iy*I; // Eqn (81)

//(14)
0 = (lh/lbar)*Lh + (le/lbar)*Le;  // Eqn (82)

//(15)
alpha*Y = ceOY*Ce + iy*I + qlLeOY*(Le - Le(-1)) + ((1/g)*by)*(B(-1) - G_Lambda) 
- ((1/r)*by)*(B - R);// Eqn (83)

//(16)
B = Thetat + (g*theta_bar*(qlLeOY/by))*(Ql(+1) + Le + G_Lambda(+1)) + 
(1 - (g*theta_bar*(qlLeOY/by)))*(Qk(+1) + K - G_Q(+1)); // Eqn (84)

//(17)
G_Z = Lambda_z + V - V(-1); // Eqn (85)
//(18)
G_Q = Lambda_q + Mu - Mu(-1); // Eqn (86)
//(19)
G_Lambda = (1/(1-(1-phi)*alpha))*G_Z + (((1-phi)*alpha)/(1-(1-phi)*alpha))*G_Q; // Eqn (87)


//Shock processes

Lambda_z = rho_z*Lambda_z(-1) + Eps_z; // Eqn (88)

V = rho_v*V(-1) + Eps_v; // Eqn (89)

Lambda_q = rho_q*Lambda_q(-1) + Eps_q; // Eqn (90)

Mu = rho_mu*Mu(-1) + Eps_mu; // Eqn (91)

Lambda_a = rho_a*Lambda_a(-1) + Eps_a; // Eqn (93)

Phi = rho_phi*Phi(-1) + Eps_phi; // Eqn (94)

Psi = rho_psi*Psi(-1) + Eps_psi; // Eqn (95)

Thetat = rho_xitheta*Thetat(-1) + Eps_xitheta; //Eqn (96)

C = (chOY/(chOY+ceOY))*Ch + (ceOY/(chOY+ceOY))*Ce;

//--- Observer equations
DLogQl = (Ql - Ql(-1) + log(g) + G_Lambda); // relative price of liquid land
DLogQ =  (G_Q + log(lambda_qbar)); // inverse of the relative price of investment
DLogC = ((chOY/(chOY+ceOY))*(Ch - Ch(-1)) + (ceOY/(chOY+ceOY))*(Ce - Ce(-1)) + log(g) + G_Lambda); //real consumption
DLogI = (I - I(-1) + log(g) + G_Lambda); //investment plus durables deflated by consumption deflator
DLogB = (B - B(-1) + log(g) + G_Lambda); //real debt growth
LogL = N + log(n); //log hours

//--- Auxillary variables
DLogGDP = iy*DLogI + (1-iy)*DLogC;
//LogGDP = LogGDP(-1) + DLogGDP;
//LogC = LogC(-1) + DLogC;
//LogI = LogI(-1) + DLogI;
//LogQl = LogQl(-1) + DLogQl;

end;

shocks;
//var Eps_a; stderr .1387; //Intermporal preference shock
//var Eps_z; stderr .0036; //Permanent neutral technology shock
//var Eps_v; stderr .0038; //transitory technology shock
//var Eps_q; stderr .0037; //Permanent shock to investment specific technical change
//var Eps_mu; stderr .0025; //Transitory shock to investment specific technical change
//var Eps_phi; stderr .0543; //housing preference shock
//var Eps_psi; stderr .0073; //Labor supply shock
//var Eps_xitheta; stderr .0126; //Collateral shock
var Eps_a; stderr 0; //Intermporal preference shock
var Eps_z; stderr 0; //Permanent neutral technology shock
var Eps_v; stderr 0; //transitory technology shock
var Eps_q; stderr 0; //Permanent shock to investment specific technical change
var Eps_mu; stderr 0; //Transitory shock to investment specific technical change
var Eps_phi; stderr 0; //housing preference shock
var Eps_psi; stderr 0; //Labor supply shock
var Eps_xitheta; stderr 0; //Collateral shock
end;

steady;
check;


//--------------------------------------------------------------------
//-- Estimation
//--------------------------------------------------------------------
varobs DLogQl DLogQ DLogC DLogI DLogB LogL;

estimated_params;
// PARAM NAME, INITVAL, LB, UB, PRIOR_SHAPE, PRIOR_P1, PRIOR_P2, PRIOR_P3, PRIOR_P4, JSCALE
// PRIOR_SHAPE: BETA_PDF, GAMMA_PDF, NORMAL_PDF, INV_GAMMA_PDF
//--- The following prior will be overwritten by DynareModifications/inverse_gamma_specificationTao.m.  
stderr Eps_a,0.1387,0.000000000001,100,INV_GAMMA2_PDF,0.01,inf;
stderr Eps_z,0.0036,0.000000000001,100,INV_GAMMA2_PDF,0.01,inf;
stderr Eps_v,0.0038,0.000000000001,100,INV_GAMMA2_PDF,0.01,inf;
stderr Eps_q,0.0037,0.000000000001,100,INV_GAMMA2_PDF,0.01,inf;
stderr Eps_mu,0.0025,0.000000000001,100,INV_GAMMA2_PDF,0.01,inf;
stderr Eps_phi,0.0543,0.000000000001,100,INV_GAMMA2_PDF,0.01,inf;
stderr Eps_psi,0.0073,0.000000000001,100,INV_GAMMA2_PDF,0.01,inf;
stderr Eps_xitheta,0.0126,0.000000000001,100,INV_GAMMA2_PDF,0.01,inf;
gamma_h, ,.00000001,.99999999999,BETA_PDF,0.333333333333333,0.235702260395516;
gamma_e, ,.00000001,.99999999999,BETA_PDF,0.333333333333333,0.235702260395516;
omega, ,.00001,10,GAMMA_PDF,2,2;
g_trans, ,.00001,10,GAMMA_PDF,0.618059245483528,0.453049079987331;
lambda_qbar_trans, ,.00001,10,GAMMA_PDF,0.618059245483528,0.453049079987331;
rho_a, ,.00000001,.99999999999,BETA_PDF,0.333333333333333,0.235702260395516;
rho_z, ,.00000001,.99999999999,BETA_PDF,0.333333333333333,0.235702260395516;
rho_v, ,.00000001,.99999999999,BETA_PDF,0.333333333333333,0.235702260395516;
rho_q, ,.00000001,.99999999999,BETA_PDF,0.333333333333333,0.235702260395516;
rho_mu,  ,.00000001,.99999999999,BETA_PDF,0.333333333333333,0.235702260395516;
rho_phi, ,.00000001,.99999999999,BETA_PDF,0.333333333333333,0.235702260395516;
rho_psi, ,.00000001,.99999999999,BETA_PDF,0.333333333333333,0.235702260395516;
rho_xitheta, ,.00000001,.99999999999,BETA_PDF,0.333333333333333,0.235702260395516;
end;
options_.blockinds = [ones(8,1) ; 2*ones(5,1) ; 3*ones(8,1)];
options_.Harvey_scale_factor = 10; //100;
//mode_file=LiuWangZha_modePOST83Fix
//stoch_simul(periods=1000,simul_seed=1,irf=24,order=1)  C I Y Ql DLogC DLogI DLogGDP DLogQl;
//shock_decomposition DLogQl	DLogQ	DLogC	DLogI	DLogB LogL;

estimated_params_init;
   //--- Mode from TZ's C/C++ program
   //gamma_e, 0.658435028; 
   //gamma_h, 0.497555761; 
   //omega, 0.175347074;  
   //g_trans, 0.422109209; 
   //lambda_qbar_trans, 1.21261837; 
   //rho_a,  0.905471687;	 
   //rho_v, 0.00947706131;	//rho_{nu_z} 
   //rho_mu, 0.294889827;	   //rho_{nu_q}
   //rho_psi,  0.982918278;	 
   //rho_q, 0.561965864;	 
   //rho_xitheta, 0.980427788;	 
   //rho_phi,  0.999758433;	 
   //rho_z, 0.426298781;	 //rho_z
   ////
   //stderr Eps_z,     0.00418908791;  
   //stderr Eps_v,   0.00365910563;  
   //stderr Eps_q,     0.00415238812;  
   //stderr Eps_mu,   0.00288990925;  
   //stderr Eps_a,     0.101281626;  
   //stderr Eps_phi,   0.0462334747;  
   //stderr Eps_psi,    0.00731583619;  
   //stderr Eps_xitheta, 0.0111777605;  
   //--- Reasonable calibrated values
   gamma_e, 0.7;	 
   gamma_h, 0.7;	 
   omega, 3.0;  
   g_trans, 0.375; 
   lambda_qbar_trans, 1.25; 
   rho_a, 0.9;	 
   rho_v, 0.9;	 
   rho_mu, 0.9;	 
   rho_psi, 0.9;	 
   rho_q, 0.9;	 
   rho_xitheta, 0.9;	 
   rho_phi, 0.9;	 
   rho_z, 0.9;	 
   //
   stderr Eps_z, 0.01;   
   stderr Eps_v, 0.01;
   stderr Eps_q, 0.01;
   stderr Eps_mu, 0.01;  
   stderr Eps_a, 0.01;
   stderr Eps_phi, 0.01;
   stderr Eps_psi, 0.01;
   stderr Eps_xitheta, 0.01;
end;


//dbstop in dynare_estimation_1 at 34;  //Setting the debuggin break point.
//============= Comments by T. Zha =================
//== mode_compute=4 is best judged by a lot of people.  5 is also good.  1 is Matlab built-in minimization (most inefficient).
//== first_obs=33 meands that we get rid of first 32 sample points. The observations (including lags and training sample) start at 33.
//==================================================
options_.nit = 10000;  //Max number of iterations; Default 1000
options_.prior_trunc = 0;
options_.crit = 1e-7;   //Adjusts convergence criteion; Default 1e-07.
options_.gradient_epsilon = 1E-06; %differentiation increment;
options_.gradient_method = 2;  //2,3,or 5 point formula used to compute gradient;  Default = 2.
options_.gradient_epsilon = 1E-06;  //differentiation increment; Default 1e-06.
estimation(datafile=data_diff_FHFA,mode_check,mode_compute=5,lik_init=2,mh_replic=0,mh_nblocks=2,presample = 3,mh_jscale=0.20,mh_drop=0.2,optim=('Display','iter','MaxFunEvals',1500),smoother,forecast=20) DLogQl  DLogQ  DLogC DLogI DLogB LogL Ql I B C Y;
                     //The last four variables  I B C Y are not needed for estimation.  Used to check the forecasts.
//estimation(datafile=CreditFrictionsDataB,mode_file=Mode021611,mode_compute=5,lik_init=2,first_obs=33,mh_replic=0,mh_nblocks=2,presample = 4,mh_jscale=0.20,mh_drop=0.2,optim=('Display','iter','MaxFunEvals',10000),smoother,forecast=20) DLogQl	DLogQ	DLogC	DLogI	DLogB LogL Ql I B C Y;
//shock_decomposition LogL Ql I B C Y;
stoch_simul(conditional_variance_decomposition=[1 4 8 16 20]) C, I, Y, N, Ql, Qk, B;
